@@ -48,10 +48,8 @@ public:
 
     int m_progress = 100;
     int m_remainingSeconds = 0;
-    int m_scannedImages = 0;
     Skanpage::ApplicationState m_state = Skanpage::SearchingForDevices;
     bool m_scanInProgress = false;
-    bool m_scanIsPreview = false;
     QRectF m_maximumScanArea;
     QRectF m_scanArea; // Rectangle from (0, 0) to (1, 1)
     Skanpage::ScanSplit m_scanSplit = Skanpage::ScanNotSplit;
@@ -61,12 +59,12 @@ public:
     QString m_deviceName;
     QString m_deviceVendor;
     QString m_deviceModel;
-    QUrl m_dumpOptionsUrl;
+    QUrl m_dumpOptionUrl;
 };
 
 using namespace KSaneCore;
 
-Skanpage::Skanpage(const QString &deviceName, const QUrl &dumpOptionsUrl, QObject *parent)
+Skanpage::Skanpage(const QString &deviceName, const QUrl &dumpOptionUrl, const QUrl &importUrl, QObject *parent)
     : QObject(parent)
     , d(std::make_unique<SkanpagePrivate>())
 {
@@ -83,12 +81,13 @@ Skanpage::Skanpage(const QString &deviceName, const QUrl &dumpOptionsUrl, QObjec
     d->m_actionCollection = new KActionCollection(this);
 
     connect(&d->m_ksaneInterface, &Interface::scannedImageReady, this, &Skanpage::imageReady);
+    connect(&d->m_ksaneInterface, &Interface::previewImageReady, this, &Skanpage::previewImageReady);
     connect(&d->m_ksaneInterface, &Interface::availableDevices, this, &Skanpage::availableDevices);
     connect(&d->m_ksaneInterface, &Interface::userMessage, this, &Skanpage::showKSaneMessage);
     connect(&d->m_ksaneInterface, &Interface::scanProgress, this, &Skanpage::progressUpdated);
     connect(&d->m_ksaneInterface, &Interface::scanFinished, this, &Skanpage::scanningFinished);
     connect(&d->m_ksaneInterface, &Interface::batchModeCountDown, this, &Skanpage::batchModeCountDown);
-    connect(&d->m_documentHandler, &DocumentModel::newPageAdded, this, &Skanpage::imageTemporarilySaved);
+    connect(&d->m_documentHandler, &DocumentModel::newPageAdded, this, &Skanpage::checkFinish);
 
     d->m_fileIOThread.start();
     d->m_documentSaver.moveToThread(&d->m_fileIOThread);
@@ -106,7 +105,11 @@ Skanpage::Skanpage(const QString &deviceName, const QUrl &dumpOptionsUrl, QObjec
     connect(&d->m_imageImport, &ImageImport::showUserMessage, this, &Skanpage::showUserMessage);
     connect(&d->m_imageImport, &ImageImport::imageImported, &d->m_documentHandler, &DocumentModel::addImage);
 
-    d->m_dumpOptionsUrl = dumpOptionsUrl;
+    d->m_dumpOptionUrl = dumpOptionUrl;
+    if (!importUrl.isEmpty()) {
+        importFile(importUrl);
+    }
+
     // try to open device from command line option first, then remembered device
     if (deviceName.isEmpty() || !openDevice(deviceName)) {
         KConfigGroup options(KSharedConfig::openStateConfig(), QStringLiteral("general"));
@@ -288,51 +291,7 @@ bool Skanpage::previewImageAvailable() const
 
 void Skanpage::previewScan()
 {
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftXOption))
-        opt->storeCurrentData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftYOption))
-        opt->storeCurrentData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::BottomRightXOption))
-        opt->storeCurrentData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::BottomRightYOption))
-        opt->storeCurrentData();
-    if (d->m_maximumScanArea.isValid())
-        setScanArea(QRectF(0.0, 0.0, 1.0, 1.0));
-
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::ResolutionOption)) {
-        opt->storeCurrentData();
-        if (QVariant minRes = opt->minimumValue(); minRes.isValid()) {
-            if (opt->type() == Option::TypeValueList)
-                opt->setValue(minRes);
-            else
-                opt->setValue(minRes.toInt() < 25 ? 25 : minRes);
-        }
-    }
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::PreviewOption))
-        opt->setValue(true);
-
-    d->m_scanIsPreview = true;
-
-    startScan();
-}
-
-void Skanpage::finishPreview()
-{
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftXOption))
-        opt->restoreSavedData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::TopLeftYOption))
-        opt->restoreSavedData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::BottomRightXOption))
-        opt->restoreSavedData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::BottomRightYOption))
-        opt->restoreSavedData();
-
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::ResolutionOption))
-        opt->restoreSavedData();
-    if (Option *opt = d->m_ksaneInterface.getOption(Interface::PreviewOption))
-        opt->setValue(false);
-
-    d->m_scanIsPreview = false;
+    d->m_ksaneInterface.startPreviewScan();
 }
 
 void Skanpage::startScan()
@@ -359,16 +318,8 @@ Skanpage::ApplicationState Skanpage::applicationState() const
 
 void Skanpage::imageReady(const QImage &image)
 {
-    if (d->m_scanIsPreview) {
-        d->m_ksaneInterface.stopScan(); // Needed for ADF
-        finishPreview();
-        d->m_previewImage = image;
-        Q_EMIT previewImageChanged(d->m_previewImage);
-        return; // Do not save the preview to disk
-    }
     if (d->m_scanSplit == ScanNotSplit && d->m_scanSubAreas.isEmpty()) {
         d->m_documentHandler.addImage(image);
-        d->m_scannedImages++;
         return; // Regular scan ends here
     }
     auto applySubAreasToImage = [&]() {
@@ -377,30 +328,36 @@ void Skanpage::imageReady(const QImage &image)
         for (const QRectF &area : d->m_scanSubAreas) {
             QImage individualImage = image.copy(toScale.mapRect(toOrigin.mapRect(area)).toRect());
             d->m_documentHandler.addImage(individualImage);
-            d->m_scannedImages++;
         }
     };
     if (!d->m_scanSubAreas.isEmpty()) { // There are sub-areas
         applySubAreasToImage();
         setScanArea(d->m_scanSubAreas.back()); // Leave scanArea as last selection
     } else {
-        bool v = d->m_scanSplit == ScanIsSplitV;
         QRectF half = d->m_scanArea;
-        if (v)
+        if (d->m_scanSplit == ScanIsSplitV) {
             half.setWidth(half.width() / 2);
-        else
+        } else {
             half.setHeight(half.height() / 2);
+        }
         d->m_scanSubAreas.append(half); // Fake sub-areas, no need for notification
-        if (v)
+        if (d->m_scanSplit == ScanIsSplitV) {
             half.moveRight(d->m_scanArea.right());
-        else
+        } else {
             half.moveBottom(d->m_scanArea.bottom());
+        }
         d->m_scanSubAreas.append(half);
         applySubAreasToImage();
     }
     clearSubAreas(); // The sub-areas last just one scan
 }
 
+void Skanpage::previewImageReady(const QImage &image)
+{
+    d->m_previewImage = image;
+    Q_EMIT previewImageChanged(d->m_previewImage);
+}
+
 void Skanpage::saveScannerOptions()
 {
     KConfigGroup options(KSharedConfig::openStateConfig(), QString::fromLatin1("Options For %1").arg(d->m_ksaneInterface.deviceName()));
@@ -460,19 +417,20 @@ bool Skanpage::openDevice(const QString &deviceName, const QString &deviceVendor
             showUserMessage(SkanpageUtils::ErrorMessage, QStringLiteral("Failed to open selected device."));
         }
     }
-    if (!d->m_dumpOptionsUrl.isEmpty()) {
+    if (!d->m_dumpOptionUrl.isEmpty()) {
         QJsonObject allScannerData;
         allScannerData[QLatin1StringView("scanner")] = d->m_ksaneInterface.scannerDeviceToJson();
         allScannerData[QLatin1StringView("options")] = d->m_ksaneInterface.scannerOptionsToJson();
         QString absolutePath;
-        if (QDir::isAbsolutePath(d->m_dumpOptionsUrl.path())) {
-            absolutePath = d->m_dumpOptionsUrl.toLocalFile();
+        if (QDir::isAbsolutePath(d->m_dumpOptionUrl.path())) {
+            absolutePath = d->m_dumpOptionUrl.toLocalFile();
         } else {
-            absolutePath = QDir::current().absoluteFilePath(d->m_dumpOptionsUrl.toLocalFile());
+            absolutePath = QDir::current().absoluteFilePath(d->m_dumpOptionUrl.toLocalFile());
         }
         QJsonDocument jsonDocument(allScannerData);
         QFile file(absolutePath);
         if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
+            qCDebug(SKANPAGE_LOG()) << "Could not open file to write option data to JSON file.";
             return false;
         }
         QTextStream out(&file);
@@ -611,14 +569,9 @@ SkanpageState *Skanpage::stateConfiguration() const
     return d->m_stateConfiguration;
 }
 
-bool Skanpage::OCRavailable() const
-{
-    return d->m_OCREngine.available();
-}
-
 void Skanpage::print()
 {
-    d->m_documentPrinter.printDocument(d->m_documentHandler.selectPages(QList<int>()));
+    d->m_documentPrinter.printDocument(d->m_documentHandler.selectPages(QList<int>()), d->m_documentHandler.activePageIndex());
 }
 
 void Skanpage::registerAction(QObject *item, QObject *shortcuts, const QString &iconText)
@@ -692,28 +645,18 @@ void Skanpage::cancelScan()
     d->m_ksaneInterface.stopScan();
 }
 
-void Skanpage::imageTemporarilySaved()
-{
-    d->m_scannedImages--;
-    checkFinish();
-}
-
 void Skanpage::scanningFinished(Interface::ScanStatus status, const QString &strStatus)
 {
     // only print debug, errors are already reported by Interface::userMessage
     qCDebug(SKANPAGE_LOG) << QStringLiteral("Finished scanning! Status code:") << status << QStringLiteral("Status message:") << strStatus;
 
-    if (d->m_scanIsPreview) { // imageReady didn't execute (there was an error)
-        finishPreview(); // Restore options anyways
-    }
-
     d->m_scanInProgress = false;
     checkFinish();
 }
 
 void Skanpage::checkFinish()
 {
-    if (d->m_scannedImages == 0 && !d->m_scanInProgress) {
+    if (d->m_documentHandler.isReady() && !d->m_scanInProgress) {
         d->m_state = ApplicationState::ReadyForScan;
         Q_EMIT applicationStateChanged(d->m_state);
     }
